;**************************************************************************
;*
;* FILE  boot.asm
;* Copyright (c) 2005, 2006 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: boot.asm,v 1.39 2007-02-27 21:34:39 tlr Exp $
;*
;* DESCRIPTION
;*   Fast C64 DTV server.
;*
;******
	processor 6502

	MAC	sac
	dc.b	$32,{1}
	ENDM
	MAC	sir
	dc.b	$42,{1}
	ENDM
	MAC	bra
_TMPBRA	set	{1}-2-.
	if	_TMPBRA < -128 || _TMPBRA > 127
	ERROR	"Branch to long!"
	endif
	dc.b	$12,_TMPBRA
	ENDM

	IFNCONST IN_ROM
DEFAULT_REG8	equ	%01010101	;all ram
	ELSE ;IN_ROM
DEFAULT_REG8	equ	%01010001	;seg1 in  rom
	ENDIF ;IN_ROM

	
action_reg		equ	$010c
store_2b		equ	$010d
store_2c		equ	$010e
store_2d		equ	$010f
store_2e		equ	$0110
store_ae		equ	$0111
store_af		equ	$0112

	
pptr_zp	equ	$fb	
	seg	code
	org	$4000

	rorg	$8000
;**************************************************************************
;*
;* NAME  reset hook ($018000 or $1f8000)
;*
;* DESCRIPTION
;*   warm start is used by the normal mem trampolines.
;*
;******
	dc.w	cold_start
	dc.w	0
dtv80_str	equ	.-$4000
	dc.b	"D"|$80,"T"|$80,"V"|$80,"8","0"

	include	"config.asm"
	
;**************************************************************************
;*
;* NAME  cold_start, warm_start
;*
;* DESCRIPTION
;*   Entry Point.
;*
;******
cold_start:
; this should already have been done, but just to be safe
	ldx	#$ff
	sei
	txs
	cld	

	IFCONST	IN_ROM
	sir	$ed
; Set $8000-$bfff mapping as $4000-$7fff mapping
	tya	; Acc=reg $e
	tax	; reg $d=Acc
	sir	$e8
	ldx	#%01000001
	jmp	cs_continue
	rend
cs_continue:
	ldy	#$02		; Map $008000 -> $8000
	ldx	#DEFAULT_REG8
	sir	$12
	ELSE ;IN_ROM
	sir	$ed
; Set $8000-$bfff mapping as $4000-$7fff mapping
	tya	; Acc=reg $e
	tax	; reg $d=Acc
	jmp	cs_continue
	rend
cs_continue:
	ldy	#$02		; Map $008000 -> $8000
	sir	$12
	ENDIF ;IN_ROM

; blank screen
	ldx	#0
	stx	$d020
	stx	$d011

; wait a while for keys to settle, and to allow new presses
;X=0
; 	ldx	#0		
	ldy	#96
lp:
	dex
	bne	lp
	dey
	bne	lp

;**************************************************************************
;*
;* NAME  non destructive key check
;*
;* DESCRIPTION
;*   Non destructive key check (SpeedDos-Plus)
;*
;*   "<stop>"             go to menu.
;*   "<-"                 take screen shot.
;*   "<ctrl>"             go to basic.
;*   ("<C=>"               go to basic with extensions, i.e turbo.)
;*   "1"                  preserve $000000-$020000 -> $080000
;*   "<left joy button>"  Check $018000 hook.
;*   "<right joy button>" load up DTVMENU/INTRO.
;*
;******
	ldx	#$00
	stx	$dc03
	dex
	ldy	$dc02
	stx	$dc02
	ldx	$dc00
	lda	#%01111111
	sta	$dc00
	lda	$dc01
	stx	$dc00
	sty	$dc02
; here we have the data from the key row in acc (just shift and bcc)
; WRITE TO PORT A               READ PORT B (56321, $DC01)
; 56320/$DC00
;          Bit 7   Bit 6   Bit 5   Bit 4   Bit 3   Bit 2   Bit 1   Bit 0
; 
; Bit 7    STOP    Q       C=      SPACE   2       CTRL    <-      1
;
; Bit 6    /       ^       =       RSHIFT  HOME    ;       *       LIRA
;
; Bit 5    ,       @       :       .       -       L       P       +
;
; Bit 4    N       O       K       M       0       J       I       9
;
; Bit 3    V       U       H       B       8       G       Y       7
;
; Bit 2    X       T       F       C       6       D       R       5
;
; Bit 1    LSHIFT  E       S       Z       4       A       W       3
;
; Bit 0    CRSR DN F5      F3      F1      F7      CRSR RT RETURN  DELETE
;
	ldx	cfg_default_action	; default action.
	lsr
	bcs	cs_skp6		; Key "1"
	jmp	preserve_lowmem
cs_skp6:
	lsr
	bcs	cs_skp3		; Key "<-"
	jmp	take_screenshot
cs_skp3:
	lsr
	bcs	cs_skp4		; Key "Ctrl"
	ldx	#ACTION_GO_BASIC
cs_skp4:
	lsr
	lsr
;	bcc	cs_skipcheck	; Key "Space"
	lsr
;	bcc	cs_commodore	; Key "C="
	lsr
	lsr
	bcs	cs_skp5		; Key "Stop"
	ldx	#ACTION_GO_MENU
cs_skp5:
	stx	action_reg
		
		
	IFNCONST IN_RAM
	ldx	#$ff
; replicate the DTV80 check at $F8F6
	stx	$dc00
	lda	$dc00
	and	#%00010000	;Left Button pressed?
	bne	cs_skp1		;no, normal reset

;yes, check for DTV80 at $018004
	sac	$ee
	lda	#$06		;Map $018000 -> $8000
	sac	$00
	ldx	#5
cs_lp3:
	lda	dtv80_str-1,x
	cmp	$8004-1,x
	bne	cs_skp2		;mismatch, skip to normal reset
	dex
	bne	cs_lp3
	ldy	#$80		;jmp ($8000)
	jmp	go_trampoline
	
cs_skp2:
	sac	$ee
	lda	#$02		;Map back $008000 -> $8000
	sac	$00	
cs_skp1:
	ENDIF ;IN_RAM

cs_skipcheck:
		
; cold start, set up dtv registers
	jsr	clone_f951
	jsr	wait_wb
		
; preform the selected action
	lda	action_reg
preform_action:
	cmp	#ACTION_NORMAL_LOAD
	bne	skp1
	jmp	start_dtvmenu
skp1:
	cmp	#ACTION_GO_BASIC
	bne	skp2
	jmp	start_basic
skp2:
	cmp	#ACTION_START_ALT_KERNAL
	bne	skp3
	jsr	start_alt_kernal
skp3:
	jmp	show_menu

;**************************************************************************
;*
;* NAME  start_alt_kernal
;*
;* DESCRIPTION
;*   start an alternate kernal at $1fe000.
;*   just returns if no kernal is present
;*
;******
start_alt_kernal:
	jsr	check_for_kernal
	beq	sak_fl1
	sei
	jsr	check_key_release
	ldx	#1
	stx	$d03f
	lda	#$1f
	sta	$d100
	dex
	stx	$d03f
	ldx	$fffc
	ldy	$fffd
	jmp	go_trampoline_abs
sak_fl1:
	rts

;**************************************************************************
;*
;* NAME  start_basic_no_new
;*
;* DESCRIPTION
;*   start basic without doing new.
;*
;******
start_basic_no_new:
	sei
	cld
	jsr	preserve_basic
	jsr	$fda3
	jsr	clone_fd50
	jsr	clone_fd15
	jsr	$ff5b

;Init Basic!
	jsr	$e453		;Init basicvectors
	jsr	$e3bf		;Init Basicram
	jsr	clone_e422	;startup message

	jsr	restore_basic

;Set stack. IMPORTANT!!
	ldx	#$fb
	txs

;initialize
	jsr	$ffcc		;CLRCH
	lda	#0
	sta	$13		;Keyb input
	jsr	$ff90   	;Program mode
	jsr	$a68e		;Point to program start

	ldx	#trampoline2_end-trampoline2-1
gbnn_lp1:
	lda	trampoline2,x
	sta	trampoline2_run,x
	dex
	bpl	gbnn_lp1

	sir	$d8		; Y mapped on reg D, and X mapped on reg 8
	jmp	trampoline2_run

trampoline2:
	rorg	$0110
trampoline2_run:
	ldy	#$01		;bank 1 ($004000)
	ldx	#%01010101	;all banks in RAM.
	sir	$12		;restore default x and y mapping.
;	jsr	$a533		;relink basic
	jsr	$a660		;clear
	jmp	$a474		;go to direct mode
	rend
trampoline2_end:
		
;**************************************************************************
;*
;* NAME  start_basic
;*
;* DESCRIPTION
;*   start normal basic
;*
;******
start_basic:
	jsr	$fda3
	jsr	clone_fd50
	jsr	clone_fd15
	jsr	$ff5b
	
	ldy	#$a0		;jmp ($a000)
; Fall Thru
go_trampoline:
	
	ldx	#trampoline_end-trampoline-1
gt_lp1:
	lda	trampoline,x
	sta	trampoline_run,x
	dex
	bpl	gt_lp1

	sty	trmp_sm2+2
	sir	$d8		; Y mapped on reg D, and X mapped on reg 8
	jmp	trampoline_run

trampoline:
	rorg	$0110
trampoline_run:
trmp_sm1:
	ldy	#$01		;bank 1 ($004000)
	ldx	#%01010101	;all banks in RAM.
	sir	$12		;restore default x and y mapping.
trmp_sm2:
	jmp	($a000)
	rend
trampoline_end:

go_trampoline_abs:
	stx	$0100
	sty	$0101
	ldy	#$01
	jmp	go_trampoline
		
;**************************************************************************
;*
;* NAME  start_dtvmenu
;*
;* DESCRIPTION
;*   start the dtvmenu (or intro) depending on configuration.
;*
;******
start_dtvmenu:
	jsr	$fda3
	jsr	clone_fd50
	jsr	clone_fd15
	jsr	$e453
	jsr	$e3bf
	jsr	$ff5b
	lda	#0
	sta	$d020
	lda	#$0b
	sta	$d011
		
	lda	#1
	tay
	tax
	jsr	$ffba

; setup filename as INTRO/DTVMENU depending on the config_byte
	lda	#intro_len
	ldx	#<intro_str
	ldy	#>intro_str
	jsr	$ffbd
	lda	config_byte
	and	#CONFIG_LOADDTVMENU
	beq	sdm_skp1
	lda	#dtvmenu_len
	ldx	#<dtvmenu_str
	ldy	#>dtvmenu_str
	jsr	$ffbd
sdm_skp1:
; execute load
	lda	#0
	jsr	$ffd5
	stx	$2d
	sty	$2e
	lda	#$93
	sta	$dd00
	lda	#$15
	sta	$d018
	jsr	wait_wb
; 	lda	#14		
; 	sta	$d020
	cli
	ldx	#$0d
	ldy	#$08
	jmp	go_trampoline_abs
	
dtvmenu_str:
	dc.b	"DTVMENU"
dtvmenu_len	equ	.-dtvmenu_str
intro_str:
	dc.b	"INTRO"
intro_len	equ	.-intro_str

;**************************************************************************
;*
;* NAME  preserve_basic/restore_basic
;*
;* DESCRIPTION
;*   show the main menu
;*
;******
preserve_basic:
	lda	$ae
	sta	store_ae	
	lda	$af
	sta	store_af	
	lda	$2b
	sta	store_2b
	lda	$2c
	sta	store_2c
	lda	$2d
	sta	store_2d
	lda	$2e
	sta	store_2e
	rts
restore_basic:
	lda	store_ae	
	sta	$ae
	lda	store_af	
	sta	$af
	lda	store_2b
	sta	$2b
	lda	store_2c
	sta	$2c
	lda	store_2d
	sta	$2d
	lda	store_2e
	sta	$2e
	rts
		
;**************************************************************************
;*
;* NAME  show_menu
;*
;* DESCRIPTION
;*   show the main menu
;*
;******
show_menu:
	jsr	clone_fd50_lite
	jsr	clone_fd15
show_menu2:
	jsr	$ff5b
show_menu3:

	lda	#$0b
	sta	$d011
	lda	#0
	sta	$d020
	lda	#6
	sta	$d021
	
	lda	#<menu_msg
	sta	pptr_zp
	lda	#>menu_msg
	sta	pptr_zp+1
	ldy	#0
sm_lp1:
	lda	(pptr_zp),y
	beq	sm_skp1
	jsr	$ffd2
	iny
	bne	sm_lp1
	inc	pptr_zp+1
	bra	sm_lp1
sm_skp1:

; fill the upper and lower parts with inverted black spaces
	ldx	#80-1
sm_lp4:
	lda	#$a0
	sta	$0400+40*0,x
	sta	$0400+40*1,x
	sta	$0400+40*23,x
	lda	$d020
	sta	$d800+40*0,x
	sta	$d800+40*1,x
	sta	$d800+40*23,x
	dex
	bpl	sm_lp4

; check for a kernal at $1fe000, or remove that option
	jsr	check_for_kernal
	bne	sm_skp2
	ldx	#40-1
	lda	#0
sm_lp5:
	sta	$d800+40*15,x
	dex
	bpl	sm_lp5
sm_skp2:
	
	lda	#$1b
	sta	$d011
sm_lp2:
	cli
sm_lp3:
	jsr	$ffe4
	beq	sm_lp3
	sei
	cmp	#"M"
	beq	go_monitor
	cmp	#"A"
	beq	go_video
	cmp	#"T"
	beq	go_dtvtrans
	cmp	#"K"
	beq	go_alt_kernal
	cmp	#"Z"
	beq	go_zero_lowmem
	cmp	#"X"
	beq	go_basic
	cmp	#"Q"
	beq	go_basic_no_new
	cmp	#136		; F7/Right Button
	beq	go_dtvmenu
	cmp	#"D"
	beq	go_dtvmenu
	jmp	sm_lp2
	
go_monitor:
	jmp	ResetEntry

go_alt_kernal:
	jsr	start_alt_kernal
	bra	sm_lp2

go_basic:
	jsr	$ff5b
	jmp	start_basic
go_basic_no_new:
	jsr	check_key_release
	jmp	start_basic_no_new

go_dtvmenu:
	jmp	start_dtvmenu
go_zero_lowmem:
	jsr	zero_lowmem
	jmp	show_menu3

go_video:
	ldx	#1
	stx	$d03f
	lda	config_byte
;	and	#CONFIG_PAL	; bit 7
	bmi	gv_skp1
	dex
gv_skp1:
	jsr	check_pal
	txa
	rol
; Now we have the supposed current config in Acc.
; Toggle the line timing bit and quit.
	eor	#%00000001
	sta	$d040
	
	ldx	#0
	stx	$d03f
	jmp	show_menu2

go_dtvtrans:
	sei
	lda	cfg_dtvtrans_port
	eor	#$ff
	jsr	server

	jmp	show_menu3

menu_msg:
	dc.b	147
	incbin	"data/menutext.bin"
	
;**************************************************************************
;*
;* NAME  check_for_kernal
;*
;* DESCRIPTION
;*   Check if there is a kernal present at $1fe000.
;*   z=1, means no kernal
;*
;******
check_for_kernal:
	sir	$e8
	ldy	#$1fe000/$4000
	ldx	#DEFAULT_REG8 & %11001111
	sir	$12
	
; is the vectors all $ff's?
	ldx	#5
	lda	#$ff
cfk_lp1:
	and	$bffa,x
	dex
	bpl	cfk_lp1

	sir	$e8
	ldy	#$02
	ldx	#DEFAULT_REG8
	sir	$12

	cmp	#$ff		;Z=1 is all $ff's.
	rts

;**************************************************************************
;*
;* NAME  check_key_release
;*
;* DESCRIPTION
;*   Check that all keys have been released.
;*
;******
check_key_release:
	lda	#%00000000
	sta	$dc00
	lda	#%11111111
ckr_lp1:
	cmp	$dc01
	bne	ckr_lp1
	rts

;**************************************************************************
;*
;* NAME  clone_e422
;*
;* DESCRIPTION
;*   Clone the basic startup print at $e422 without the new at the end.
;*
;******
clone_e422:
	lda	$2b
	ldy	$2c
	jsr	$a408
	lda	#$73
	ldy	#$e4
	jsr	$ab1e
	lda	$37
	sec
	sbc	$2b
	tax
	lda	$38
	sbc	$2c
	jsr	$bdcd
	lda	#$60
	ldy	#$e4
	jmp	$ab1e

;**************************************************************************
;*
;* NAME  clone_fd50, clone_fd50_lite
;*
;* DESCRIPTION
;*   Clone the memory init at $fd50.
;*
;******
clone_fd50_lite:
	lda	#0
	sta	$c6
	sta	$c7
	ldx	#14
cfd_lp1:
	sta	$8c,x
	sta	$d0,x
	dex
	bpl	cfd_lp1

	jsr	$fda3
	jsr	$e453	;InitBasic

	ldx	#$c0
	stx	$9d	;Status
	lda	$030f
	and	#$f7
	sta	$030f
	ldy	#0
	sty	$02a1	
	sty	$0281

	bra	cfd50_ex1

clone_fd50:
	lda	#0
	tay
cfd50_lp1:
	sta	$0002,y
	sta	$0200,y
	sta	$0300,y
	iny
	bne	cfd50_lp1

cfd50_ex1:
	ldx	#$03
	lda	#$3c
	sta	$b2
	stx	$b3

	ldx	#$00
	ldy	#$a0
	jmp	$fd8c	;Set MemBounds

;**************************************************************************
;*
;* NAME  clone_fd15, clone2_fd15
;*
;* DESCRIPTION
;*   Clone the vector setup at $fd15.
;*
;******
; clone fd15, without killing $0330/$0331/$0332/$0333.
clone2_fd15:
	ldy	#$1b
	dc.b	$2c
; clone normal fd15
clone_fd15:
	ldy	#$1f
cfd15_lp1:
	lda	$fd30,y
	sta	$0314,y
	dey
	bpl	cfd15_lp1
	rts

;**************************************************************************
;*
;* NAME  clone_f951
;*
;* DESCRIPTION
;*   Clone of the DTV video init at $f951.
;*
;******
clone_f951:
	ldx	#$01
	stx	$d03f

; is the config all $ff's?
	ldx	#5
	lda	#$ff
cri_lp1:
	and	cfg_hard_video,x
	dex
	bpl	cri_lp1
	cmp	#$ff
	bne	clone_f951_hard	; no, skip to hard video
; yes, do normal config

	lda	#$00
	sta	$dd03
	lda	$dd01
	sta	$aa
	and	#$03
	sta	$d040
	lda	$aa
	lsr
	lsr
	and	#$0f
	sta	$d04f
	bit	$aa
	bvs	cri_skp1
	lda	#7
	dc.b	$2c
cri_skp1:
	lda	#6
	sta	$d04e
	bit	$aa
	bmi	cri_skp2
	ldy	#$02
	dc.b	$2c
cri_skp2:
	ldy	#$05
	lda	$01
	and	#$10
	bne	cri_skp3
	tya
	clc
	adc	#$06
	tay
	lda	#$00
	sta	$d04e
cri_skp3:
	ldx	#$2
cri_lp2:
	lda	consts,y
	sta	$d041,x
	dey
	dex
	bpl	cri_lp2
cri_ex1:
	ldx	#$00
	stx	$d03f
	rts
consts:
	dc.b	$1c,$13,$2a
	dc.b	$24,$31,$5b
	dc.b	$1c,$00,$00
	dc.b	$24,$00,$00

clone_f951_hard:
	ldx	#3
cfh_lp1:
	lda	cfg_hard_video,x
	sta	$d040,x
	dex
	bpl	cfh_lp1
	ldx	#1
cfh_lp2:
	lda	cfg_hard_video+4,x
	sta	$d04e,x
	dex
	bpl	cfh_lp2
	bra	cri_ex1

;**************************************************************************
;*
;* wait for vertical blanking
;*
;******
wait_wb:
ww_lp1:
	lda	$d011
	bpl	ww_lp1
ww_lp2:
	lda	$d011
	bmi	ww_lp2
	rts

;**************************************************************************
;*
;* NAME  check_pal
;*
;* DESCRIPTION
;*   Check if pal.
;*   C=1 means pal
;*
;*   
;******
check_pal:
;Wait for the negative part of the screen
cp_lp1:
	lda	$d011
	bmi	cp_lp1
cp_lp2:
	lda	$d011
	bpl	cp_lp2
;check for pal
cp_lp3:
	lda	$d012
	cmp	#$20
	beq	cp_ex1		; C=1
	lda	$d011
	bmi	cp_lp3
	clc
cp_ex1:
	rts

	
;**************************************************************************
;*
;* NAME  clear_dma, do_dma
;*
;* DESCRIPTION
;*
;******
clear_dma:
	ldx	#$1f
	lda	#0
cdm_lp1:
	sta	$d300,x
	dex
	bpl	cdm_lp1
	rts

do_dma:
	lda	#%00001101
	sta	$d31f
ddm_lp1:
	lda	$d044
	lsr
	lsr
	sta	$d020
	lda	$d31f
	lsr
	bcs	ddm_lp1
	rts

;**************************************************************************
;*
;* NAME  zero_lowmem
;*
;* DESCRIPTION
;*   Zero fill $000000 -> $010000
;*
;******
zero_lowmem:
	ldy	#1
	sty	$d03f
	jsr	clear_dma
	sta	$0800
; dest step = 1
	sty	$d308
; $0800 - $10000
	lda	#$08
	sta	$d301
	sta	$d304
	lda	#$40
	sta	$d302
	sta	$d305
	lda	#$f8
	sta	$d30b
	jsr	do_dma
	jsr	clear_dma
	ldy	#0
	sty	$d020
	sty	$d03f
	rts


;**************************************************************************
;*
;* NAME  preserve_lowmem
;*
;* DESCRIPTION
;*   Copy $000000-$020000 -> $100000
;*   use no stack.
;*
;******
preserve_lowmem:
	ldy	#1
	sty	$d03f
	lda	#0
	ldx	#$1f
pl_lp1:
	sta	$d300,x
	dex
	bpl	pl_lp1
; step = 1
	sty	$d306
	sty	$d308
	lda	#$40
	sta	$d302
	lda	#$50
	sta	$d305

	lda	#%00001101
	sta	$d31f
pl_lp2:
	lda	$d044
	lsr
	lsr
	sta	$d020
	lda	$d31f
	lsr
	bcs	pl_lp2
; stack area now copied, stack may be used from here
	lda	#$41
	sta	$d302
	lda	#$51
	sta	$d305
	jsr	do_dma
	jsr	clear_dma
	ldy	#0
	sty	$d020
	sty	$d03f
	
	lda	#ACTION_GO_MENU
	sta	action_reg
	jmp	cs_skipcheck
	

;**************************************************************************
;*
;* NAME  take_screenshot
;*
;* DESCRIPTION
;*   Create a screen shot at $0801.
;*
;******
BASE	equ	$0000
take_screenshot:
	ldx	#show_screenshot_end-show_screenshot
tss_lp1:
	lda	show_screenshot_st,x
	sta	show_screenshot+BASE,x
	dex
	bpl	tss_lp1
	
	ldx	#0
tss_lp2:
	lda	$0400,x
	sta	screen_st+BASE,x
	lda	$0500,x
	sta	screen_st+BASE+$0100,x
	lda	$0600,x
	sta	screen_st+BASE+$0200,x
	lda	$06e8,x
	sta	screen_st+BASE+$02e8,x
	lda	$d800,x
	sta	color_st+BASE,x
	lda	$d900,x
	sta	color_st+BASE+$0100,x
	lda	$da00,x
	sta	color_st+BASE+$0200,x
	lda	$dae8,x
	sta	color_st+BASE+$02e8,x
	inx
	bne	tss_lp2

	lda	#<[show_screenshot+1]
	sta	$2b
	lda	#>[show_screenshot+1]
	sta	$2c
	lda	#<show_screenshot_full_end
	sta	$2d
	sta	$ae
	lda	#>show_screenshot_full_end
	sta	$2e
	sta	$af

; tweak
	jsr	clone_f951
	jsr	clone_fd50_lite
	jsr	clone_fd15
	jsr	$ff5b
	lda	#$60
	sta	sss_sm4
	jsr	sss_test
	lda	#$4c
	sta	sss_sm4
tss_lp3:	
	cli
	lda	sss_sm1+1
	sta	$d020
	ldx	sss_sm2+1
	stx	$d021
	ldy	ctab,x
	ldx	#39
tss_lp5:
	lda	shot_text,x
	and	#$3f
	beq	tss_skp1
	sta	$0400,x
	tya
	sta	$d800,x
tss_skp1:
	dex
	bpl	tss_lp5

	jsr	$ffe4
	tay
	cpy	#133 		;F1
	bne	tss_skp2
	ldx	sss_sm1+1
	inx
	txa
	and	#$0f
	sta	sss_sm1+1
tss_skp2:
	cpy	#134 		;F3
	bne	tss_skp3
	ldx	sss_sm2+1
	inx
	txa
	and	#$0f
	sta	sss_sm2+1
tss_skp3:
	cpy	#3
	bne	tss_lp3

	lda	$d018
	and	#%11111110
	sta	sss_sm3+1
	sei
	lda	#ACTION_GO_MENU
	sta	action_reg
	jmp	cs_skipcheck
	
ctab:
	dc.b	1,0,1,0,0,0,1,0
	dc.b	0,1,0,1,1,0,0,0
shot_text:
	dc.b	"$D020=F1 $D021=F3@@@@@@@@@@@@@@@EXIT=R/S"
	
show_screenshot_st:
	rorg	$0800
show_screenshot:
	dc.b	0
	dc.w	sss_endline
	dc.w	0
	dc.b	$9e,"2069 /T.L.R/",0
;	        0 SYS2069 /T.L.R/
sss_endline:
	dc.w	0
	
	sei
sss_test:
sss_sm1:
	lda	#14
	sta	$d020
sss_sm2:
	lda	#6
	sta	$d021
sss_sm3:
	lda	#$14
	sta	$d018
	ldx	#0
sss_lp1:
	lda	screen_st,x
	sta	$0400,x
	lda	screen_st+$0100,x
	sta	$0500,x
	lda	screen_st+$0200,x
	sta	$0600,x
	lda	screen_st+$02e8,x
	sta	$06e8,x
	lda	color_st,x
	sta	$d800,x
	lda	color_st+$0100,x
	sta	$d900,x
	lda	color_st+$0200,x
	sta	$da00,x
	lda	color_st+$02e8,x
	sta	$dae8,x
	inx
	bne	sss_lp1
sss_lp2:
sss_sm4:
	jmp	sss_lp2
	
show_screenshot_end:
screen_st:
color_st			equ	screen_st+$03e8
show_screenshot_full_end	equ	color_st+$03e8
	rend
		
;**************************************************************************
;*
;* external code
;*
;******
	include	"monitor.asm"
store_d020	equ	$0140
store_reg_8	equ	$0141
store_reg_e	equ	$0142
store_reg_d	equ	$0143
	include	"server.asm"
; eof
